@@ -132,10 +132,9 @@ TSeries::TSeries(const Time& t0, const Interval& dt, size_type NData,
     mData.reset(DVptr);
 
     //----------------------------------  Loop over time points
-    Time t = mT0;
     for (size_type i=0 ; i<NData ; i++) {
+	Time t = mT0 + mDt * double(i);
         (*DVptr)[i] = funct.Tspace(t);
-        t += mDt;
     }
 }
 
@@ -711,9 +710,11 @@ TSeries::operator/(const TSeries& rhs) const {
     TSeries r(mT0, mDt, mData->Extract(inx, nw));
     switch(mData->getType()) {
     case DVector::t_short:
+        r.Convert(DVector::t_float);
+	break;
     case DVector::t_int:
     case DVector::t_long:
-        r.Convert(DVector::t_float);
+        r.Convert(DVector::t_double);
 	break;
     default:
         break;
