@@ -1,5 +1,4 @@
-VCSID("@(#)$Id: confinfo.c 7338 2015-04-14 23:24:47Z ed.maros@LIGO.ORG $");
-
+VCSID("@(#)$Id: confinfo.c 7389 2015-06-11 18:51:56Z james.batch@LIGO.ORG $");
 /*----------------------------------------------------------------------*/
 /*                                                         		*/
 /* Module Name: confinfo						*/
@@ -34,6 +33,7 @@ VCSID("@(#)$Id: confinfo.c 7338 2015-04-14 23:24:47Z ed.maros@LIGO.ORG $");
 #else
 #include <stdio.h>
 #include <sys/types.h>
+#include <netdb.h>	/* For gethostbyname() */
 #include <sys/socket.h>
 #include <pthread.h>
 #include <netinet/in.h>
@@ -47,7 +47,10 @@ VCSID("@(#)$Id: confinfo.c 7338 2015-04-14 23:24:47Z ed.maros@LIGO.ORG $");
 #include "dtt/confinfo.h"
 #include "dtt/gdsndshost.h"
 #include "dtt/gdsmain.h" // test to see if __init__ gets defined correctly.
+#define _RPC_HDR	 // To get values from rpcinc.h
+#include "dtt/rpcinc.h"  // For RPC_PROGNUM_AWG and RPC_PROGNUM_TESTPOINT
 #include "dtt/rmorg.h"   // Need for TP_MAX_NODE. */
+#include "dtt/gdsreadtppar.h"
 
 /*----------------------------------------------------------------------*/
 /*                                                         		*/
@@ -187,6 +190,8 @@ static int my_debug = 0 ;
       char*		q;		/* temp. ptr to handle m.line */
       char*		qq;		/* temp. ptr to handle m.line */
       char*		sender;		/* sender's address */
+      char*		env_val ;	/* Pointer to return from getenv() */
+
    
       if ((id != 0) || ((_MAX+1) * sizeof (char*) >= len)) {
          return NULL;
@@ -231,119 +236,265 @@ static int my_debug = 0 ;
 	    *p = b ; /* Point to the string just written. */
 	    *(++p) = NULL ; /* Increment the string pointer and make the new location null */
 	    ++num ; /* Increment the count of responses. */
-	    left -= strlen(b) + 1 ; /* There's a \0 at the end. */
-	    b += strlen(b) + 1 ; /* Point beyond the \0 at the end. */
+	    left -= strlen(b) + 1 ; /* There's a '\0' at the end. */
+	    b += strlen(b) + 1 ; /* Point beyond the '\0' at the end. */
 	 }
       }
-      /* create a socket */
-      sock = socket (PF_INET, SOCK_DGRAM, 0);
-      if (sock == -1) {
-         return NULL;
-      }
-      /* enable broadcast */
-      nset = 1;
-      if (setsockopt (sock, SOL_SOCKET, SO_BROADCAST, 
-      (char*) &nset, sizeof (nset)) == -1) {
-         close (sock);
-         return NULL;
-      }
-   
-      /* bind socket */
-      name.sin_family = AF_INET;
-      name.sin_port = 0;
-      name.sin_addr.s_addr = htonl (INADDR_ANY);
-      if (bind (sock, (struct sockaddr*) &name, sizeof (name))) {
-         close (sock);
-         return NULL;
-      }
 
-      /* set broadcast address */
-      /* Determine if the broadcast query to the awgtpmans goes to a 
-       * special network.  This is indicated by the environment variable
-       * LIGORTNETIP being set. If it isn't, just blast it out the default.
-       */
+      if ((env_val = getenv("LIGO_INJ_MODEL")) != (char *) NULL) 
       {
-	 char *bcast_net ;
+	 modelInfo node[TP_MAX_NODE] ;
+	 int	   nread = 0 ;
+	 int 	   i ;
+	 char	  *pch ;
 
-	 if ((bcast_net = getenv("LIGO_RT_BCAST")) != (char *) NULL &&
-	    (inet_aton(bcast_net, &name.sin_addr.s_addr) == 1))
+	 /* Get the list of awg and tp servers from the LIGO_INJ_MODEL env. variable value. */
+	 /* Look in the testpoint.par file, found in $CDSROOT/target/gds/param/testpoint.par */
+	 if (readTestpointPar(node, &nread, TP_MAX_NODE) == 0)
 	 {
-	    if (my_debug) fprintf(stderr, "getConfInfo_r() - Setting broadcast address to %s\n", bcast_net) ;
+	    if (my_debug)
+	    {
+	       fprintf(stderr, "Number read is %d\n", nread) ;
+	       for (i = 0; i < nread; i++)
+	       {
+		  fprintf(stderr, "%d: dcuid= %d, system= %s\n", i, node[i].dcuid, node[i].modelname) ;
+	       }
+	    }
+
+	    /* env_val should contain 1 or more model names separated by commas.  For each name,
+	     * find the corresponding dcuid and hostname in the nodes read from the testpoint.par file
+	     * and construct a string for the RPC address for the awg and testpoint managers.
+	     */
+	    pch = strtok(env_val, ",") ;
+	    while (pch != (char *) NULL)
+	    {
+	       if (my_debug) fprintf(stderr, "Locating system for model %s\n", pch) ;
+
+	       /* Find the model name in the nodes found in the testpoint.par file. */
+	       for (i = 0; i < nread; i++)
+	       {
+		  if (strcmp(node[i].modelname, pch) == 0)
+		  {
+		     /* Found the node. */
+		     struct hostent	*host ;
+		     char		ipstr[INET_ADDRSTRLEN] ;
+
+		     /* We have a host name, but we want the IP address. */
+		     host = gethostbyname(node[i].hostname) ;
+		     if (!host)
+			break ;
+		     (void) inet_ntop(host->h_addrtype, *(host->h_addr_list), ipstr, sizeof(ipstr)) ;
+
+		     /* Construct a string for the awg. */
+		     sprintf(b, "awg %d 0 %s %d 1 127.0.0.1", node[i].dcuid, ipstr, RPC_PROGNUM_AWG + node[i].dcuid) ;
+		     /* Fix up p, b, and left. */
+		     *p = b ; 		/* Point to the string just written. */
+		     *(++p) = NULL ; 	/* Increment the string pointer and make the new location null. */
+		     ++num ;		/* Increment the count of responses. */
+		     left -= strlen(b)+1 ; /* There's a '\0' at the end. */
+		     b += strlen(b) + 1 ; /* Point beyond the '\0' and the end. */
+
+		     sprintf(b, "tp %d 0 %s %d 1 127.0.0.1", node[i].dcuid, ipstr, RPC_PROGNUM_TESTPOINT + node[i].dcuid) ;
+		     /* Fix up p, b, and left. */
+		     *p = b ; 		/* Point to the string just written. */
+		     *(++p) = NULL ; 	/* Increment the string pointer and make the new location null. */
+		     ++num ;		/* Increment the count of responses. */
+		     left -= strlen(b)+1 ; /* There's a '\0' at the end. */
+		     b += strlen(b) + 1 ; /* Point beyond the '\0' and the end. */
+		     break ;
+		  }
+	       }
+	       /* There may be another... */
+	       pch = strtok(NULL, ",") ;
+	    }
+	 }
+	 
+      }
+      else
+      {
+	 /* Get the list of awg and tp servers by broadcasting for them. */
+
+	 /* create a socket */
+	 sock = socket (PF_INET, SOCK_DGRAM, 0);
+	 if (sock == -1) {
+	    return NULL;
+	 }
+	 /* enable broadcast */
+	 nset = 1;
+	 if (setsockopt (sock, SOL_SOCKET, SO_BROADCAST, 
+	 (char*) &nset, sizeof (nset)) == -1) {
+	    close (sock);
+	    return NULL;
+	 }
+      
+	 /* bind socket */
+	 name.sin_family = AF_INET;
+	 name.sin_port = 0;
+	 name.sin_addr.s_addr = htonl (INADDR_ANY);
+	 if (bind (sock, (struct sockaddr*) &name, sizeof (name))) {
+	    close (sock);
+	    return NULL;
+	 }
+	 name.sin_port = (short) htons (_MAGICPORT);
+      
+
+	 /* set broadcast address */
+	 /* Determine if the broadcast query to the awgtpmans goes to a 
+	  * special network.  This is indicated by the environment variable
+	  * LIGO_RT_BCAST being set. If it isn't, just blast it out the default.
+	  * Allow LIGO_RT_BCAST to be a comma separated list of broadcast addresses.
+	  */
+	 if ((env_val = getenv("LIGO_RT_BCAST")) != (char *) NULL)
+	 {
+	    char	*pch = strtok(env_val, ",") ;
+
+	    while (pch != (char *) NULL)
+	    {
+	       if (inet_aton(pch, &name.sin_addr.s_addr) != 0)
+	       {
+		  if (my_debug) fprintf(stderr, "getConfInfo_r() - Setting broadcast address to %s\n", pch) ;
+	       }
+
+	       /* send request, which consists of 0000. */
+	       *((long*) buf) = htonl (0);
+	       nbytes = sendto (sock, buf, 4, 0, (struct sockaddr*) &name, sizeof (struct sockaddr_in));
+	       if (nbytes < 0) 
+	       {
+		  close (sock);
+		  return NULL;
+	       }
+	 
+	       /* wait for answers */
+	       start = TAInow();
+	       if (timeout < 0) {
+		  timeout = fabs (timeout);
+	       } 
+	       if (fabs (timeout) < 1E-9) {
+		  timeout = _DEFAULT_TIMEOUT;
+	       }
+	       while (((diff = start + (tainsec_t)(timeout * 1E9) - TAInow()) > 0) && (num < _MAX)) 
+	       {
+		  /* poll socket */
+		  FD_ZERO (&readfds);
+		  FD_SET (sock, &readfds);
+		  wait.tv_sec = diff / _ONESEC;
+		  wait.tv_usec = (diff % _ONESEC) / 1000;
+		  nset = select (FD_SETSIZE, &readfds, 0, 0, &wait);
+		  if (nset < 0) {
+		     close (sock);
+		     return NULL;
+		  }
+		  else if (nset == 0) {
+		     break;
+		  }
+		  /* get answer */
+		  size = sizeof (struct sockaddr_in);
+		  nbytes = recvfrom (sock, buf, sizeof (buf) - 1, 0,
+		     (struct sockaddr*) &name, &size);
+		  sender = inet_ntoa (name.sin_addr);
+		  buf[nbytes] = 0;
+		  /*printf("Received `%s' from %s\n", buf, sender);*/
+		  if ((nbytes >= 4) && (ntohl(*((long*) buf)) == 0)) 
+		  {
+		     if (left < strlen (buf + 4) + 1) {
+			return NULL;
+		     }
+		     q = buf + 4;
+		     do 
+		     {
+			qq = strchr (q, '\n');
+			if (qq != NULL) *qq = 0;
+			if (left < strlen (q) + strlen (sender) + 2) {
+			   return NULL;
+			}
+			sprintf (b, "%s %s", q, sender);
+			if (num < _MAX) {
+			   *p = b;
+			   *(++p) = NULL;
+			   ++num;
+			}
+			left -= strlen (b) + 1;
+			b += strlen (b) + 1;
+			q = (qq == NULL) ? q + strlen (q) : qq + 1;
+		     } while (*q);
+		  }
+	       }
+	       pch = strtok(NULL, ",") ;
+	    }
 	 }
 	 else
 	 {
+	    /* No LIGO_RT_BCAST variable found, send to local broadcast address. */
 	    if (my_debug) fprintf(stderr, "getConfInfo_r() - Setting broadcast address to local broadcast\n") ;
 	    name.sin_addr.s_addr = htonl(INADDR_BROADCAST) ;
-	 }
-      }
-      name.sin_family = AF_INET;
-      name.sin_port = (short) htons (_MAGICPORT);
-   
-      /* send request, which consists of 0000. */
-      *((long*) buf) = htonl (0);
-      nbytes = sendto (sock, buf, 4, 0, (struct sockaddr*) &name, sizeof (struct sockaddr_in));
-      if (nbytes < 0) 
-      {
-         close (sock);
-         return NULL;
-      }
-   
-      /* wait for answers */
-      start = TAInow();
-      if (timeout < 0) {
-         timeout = fabs (timeout);
-      } 
-      if (fabs (timeout) < 1E-9) {
-         timeout = _DEFAULT_TIMEOUT;
-      }
-      while (((diff = start + (tainsec_t)(timeout * 1E9) - TAInow()) > 0) && (num < _MAX)) 
-      {
-         /* poll socket */
-         FD_ZERO (&readfds);
-         FD_SET (sock, &readfds);
-         wait.tv_sec = diff / _ONESEC;
-         wait.tv_usec = (diff % _ONESEC) / 1000;
-         nset = select (FD_SETSIZE, &readfds, 0, 0, &wait);
-         if (nset < 0) {
-            close (sock);
-            return NULL;
-         }
-         else if (nset == 0) {
-            break;
-         }
-         /* get answer */
-         size = sizeof (struct sockaddr_in);
-         nbytes = recvfrom (sock, buf, sizeof (buf) - 1, 0,
-            (struct sockaddr*) &name, &size);
-         sender = inet_ntoa (name.sin_addr);
-         buf[nbytes] = 0;
-	 /*printf("Received `%s' from %s\n", buf, sender);*/
-         if ((nbytes >= 4) && (ntohl(*((long*) buf)) == 0)) 
-	 {
-            if (left < strlen (buf + 4) + 1) {
-               return NULL;
-            }
-            q = buf + 4;
-            do 
+
+	    /* send request, which consists of 0000. */
+	    *((long*) buf) = htonl (0);
+	    nbytes = sendto (sock, buf, 4, 0, (struct sockaddr*) &name, sizeof (struct sockaddr_in));
+	    if (nbytes < 0) 
 	    {
-               qq = strchr (q, '\n');
-               if (qq != NULL) *qq = 0;
-               if (left < strlen (q) + strlen (sender) + 2) {
-                  return NULL;
-               }
-               sprintf (b, "%s %s", q, sender);
-               if (num < _MAX) {
-                  *p = b;
-                  *(++p) = NULL;
-                  ++num;
-               }
-               left -= strlen (b) + 1;
-               b += strlen (b) + 1;
-               q = (qq == NULL) ? q + strlen (q) : qq + 1;
-            } while (*q);
-         }
+	       close (sock);
+	       return NULL;
+	    }
+	 
+	    /* wait for answers */
+	    start = TAInow();
+	    if (timeout < 0) {
+	       timeout = fabs (timeout);
+	    } 
+	    if (fabs (timeout) < 1E-9) {
+	       timeout = _DEFAULT_TIMEOUT;
+	    }
+	    while (((diff = start + (tainsec_t)(timeout * 1E9) - TAInow()) > 0) && (num < _MAX)) 
+	    {
+	       /* poll socket */
+	       FD_ZERO (&readfds);
+	       FD_SET (sock, &readfds);
+	       wait.tv_sec = diff / _ONESEC;
+	       wait.tv_usec = (diff % _ONESEC) / 1000;
+	       nset = select (FD_SETSIZE, &readfds, 0, 0, &wait);
+	       if (nset < 0) {
+		  close (sock);
+		  return NULL;
+	       }
+	       else if (nset == 0) {
+		  break;
+	       }
+	       /* get answer */
+	       size = sizeof (struct sockaddr_in);
+	       nbytes = recvfrom (sock, buf, sizeof (buf) - 1, 0,
+		  (struct sockaddr*) &name, &size);
+	       sender = inet_ntoa (name.sin_addr);
+	       buf[nbytes] = 0;
+	       /*printf("Received `%s' from %s\n", buf, sender);*/
+	       if ((nbytes >= 4) && (ntohl(*((long*) buf)) == 0)) 
+	       {
+		  if (left < strlen (buf + 4) + 1) {
+		     return NULL;
+		  }
+		  q = buf + 4;
+		  do 
+		  {
+		     qq = strchr (q, '\n');
+		     if (qq != NULL) *qq = 0;
+		     if (left < strlen (q) + strlen (sender) + 2) {
+			return NULL;
+		     }
+		     sprintf (b, "%s %s", q, sender);
+		     if (num < _MAX) {
+			*p = b;
+			*(++p) = NULL;
+			++num;
+		     }
+		     left -= strlen (b) + 1;
+		     b += strlen (b) + 1;
+		     q = (qq == NULL) ? q + strlen (q) : qq + 1;
+		  } while (*q);
+	       }
+	    }
+	 }
+	 close (sock);
       }
-      close (sock);
    
       /* sort answeres */
       qsort (buffer, num, sizeof (char*), 
