@@ -1,4 +1,4 @@
-/* version $Id: testchn.cc 6311 2010-09-17 17:05:03Z james.batch@LIGO.ORG $ */
+/* version $Id: testchn.cc 7413 2015-07-17 22:23:49Z ed.maros@LIGO.ORG $ */
 /*----------------------------------------------------------------------*/
 /*                                                         		*/
 /* Module Name: testchn							*/
@@ -19,7 +19,7 @@ namespace diag {
    bool channelHandler::setSiteIfo (char SiteDefault, char SiteForce, 
                      char IfoDefault, char IfoForce)
    {
-      siteDefault = siteDefault;
+      siteDefault = SiteDefault;
       siteForce = SiteForce;
       ifoDefault = IfoDefault;
       ifoForce = IfoForce;
