@@ -1,4 +1,4 @@
-VCSID("$(#)$Id: awggui.cc 7380 2015-05-22 22:36:55Z john.zweizig@LIGO.ORG $");
+VCSID("$(#)$Id: awggui.cc 7389 2015-06-11 18:51:56Z james.batch@LIGO.ORG $");
 /*---------------------------------------------------------------------------*/
 /*                                                                           */
 /* Module Name:  awggui                                                      */
@@ -171,14 +171,14 @@ VCSID("$(#)$Id: awggui.cc 7380 2015-05-22 22:36:55Z john.zweizig@LIGO.ORG $");
    "Version 2.17.1, May 2015",
    "",
    "* Fix Bugzilla 851 to add ability to enter numbers smaller than 1e-6 by",
-   "  changing the format of floating point numbers from %%f to %%e when ",
+   "  changing the format of floating point numbers from %f to %e when ",
    "  commands are sent to the awg.  There are still limits to the values",
    "  the awg will accept, but awggui doesn't validate the values entered ",
    "  by the user.",
    "",
    "Version 2.16.17.1, January 2015",
    "",
-   "* Embed change notes into code to eliminate the dependence on an"
+   "* Embed change notes into code to eliminate the dependence on an",
    "  external change notes file",
    "",
    "Version 2.16.12, March 2014",
