@@ -483,7 +483,7 @@ namespace dfm {
          fFVers->Select (framefast::kDefaultFrameVersion);
          // CFType->AddEntry ("binary", 3);
          // CFType->AddEntry ("text", 4);
-         AddFrame (fFVers, fL[6]);
+         AddFrame (fFVers, fL[1]); // Was fL[6] which is an error, there's only 4 of them.
       }
       // fifth line
       if (fSourceSel && fTimeFormatsel) {
