@@ -12,6 +12,7 @@
 // DEBUG = 3: verbose and random packet receiving errors
 // #define DEBUG 1
 
+//  PBTEASEPACKET0 defined in framexmittypes
 
 #include "PConfig.h"
 #include <cstdio>
@@ -182,6 +183,16 @@ namespace framexmit {
       }
       pkts.clear();
       first = true;
+#if defined(PBTEASEPACKET0)
+      if(par.teasePacket0) {
+	 lastSequenceSuccessful = false;
+	 drawOutFrameCount = 0;
+	 lastFrameDrawnOut = 0; // Sequence 0 should normally not be drawn out, since a blocking
+	                        // call is always called when receiving the first frame
+	 currentFrameDrawingOut = 0; // The current frame we are working on
+	 lastFrameDuration = par.lastFrameDurationDefault; // duration of last frame defaults to 4 seconds if no frame seen
+      }
+#endif // PBTEASEPACKET0
 
       return true;
    }
@@ -262,6 +273,20 @@ namespace framexmit {
       int64_t lastTimeSequenceDone = 0;	// the last time a sequence finished
       int64_t firstPktTime = 0;	// when the first packet arrived
 
+#if defined(PBTEASEPACKET0)
+      if(par.teasePacket0) {
+	 searchForNewseqStartTime = get_timestamp();
+	 lastRebroadcastTime = 0;  // can only issue new request if this is at zero
+	 retryDrawOut = 0;		// # of rebroadcasts
+	 drawOutFrameCount = 0;            // reset the number of frames drawn out to zero
+	 if(first) {
+	    currentFrameDrawingOut = oldseq; // This is the current frame we are drawing out
+	 }
+	 else {
+	    currentFrameDrawingOut = oldseq + 1; // This is the current frame we are drawing out
+	 }
+      }
+#endif // PBTEASEPACKET0
 
       // check socket
       if (sock < 0) {
@@ -273,6 +298,11 @@ namespace framexmit {
          maxlen = -1;
       }
       else if (maxlen <= 0) {
+#if defined(PBTEASEPACKET0)
+      if(par.teasePacket0) {
+	 lastSequenceSuccessful = false;
+      }
+#endif // PBTEASEPACKET0
          return -2;
       }
    
@@ -292,10 +322,60 @@ namespace framexmit {
    
       // read a packet if buffer empty
       if (pkts.size() == 0) {
+#if defined(PBTEASEPACKET0)
+	 if(par.teasePacket0) {
+	 	////////////////////////////////////////////////////////////
+	 	// On first socket read, don't just block (unless we've never seen
+	 	// a sequence or the last sequence was unsuccessful):
+	 	// Wait for 4 seconds, after that issue a rebroadcast
+	 	// request for packet 0 of what we expect to be the new sequence
+	 	// (oldseq + 1)
+	 	// This routine will stop as soon as ANY packet arrives, even
+	 	// if from a different sequence, at which point the code will
+	 	// go to the main packet processing loop further below.
+	 	// This code just says that SOMETHING should have been received
+	 	// in the first four seconds, and if it isn't, then request
+	 	// packet 0 of what we expected to be the next sequence.
+	 	//
+	 	////////////////////////////////////////////////////////////
+	 	// HOWEVER: DO block as long as necessary if we've not recently
+	 	// seen a successful sequence
+	 	if(lastSequenceSuccessful == false) {
+	 	   // request a packet
+	 	   if (!getPacket()) {
+	 	      lastSequenceSuccessful = false;
+	 	      return -3;
+	 	   }
+	 	}
+	 	// Next code run only if we recently saw a successful sequence
+	 	// so that we know that the next packets should be arriving in 4 seconds
+	 	else {
+         	    // request a packet
+         	    while (getPacket (false)) {
+	 		// Has it now been too long? Should we issue a rebroadcast request?
+	 		int64_t now = get_timestamp();
+	 	
+	 		int r = teasePacket0(now, 1);
+	 		if(r != 0) return(r);
+	 	
+	 		// Wait a little bit before checking again
+	 		timespec wait = {(1000LL * rcvDelayTick) / 1000000000LL, 
+	 				 (1000LL * rcvDelayTick) % 1000000000LL};
+	 		nanosleep (&wait, 0);
+	 	    } // while getPacket
+	 	}
+	 } // if(par.teasePacket0)
+	 else {
+	    if (!getPacket()) {
+	       return -3;
+	    }
+	 } // if(par.teasePacket0)
+#else // PBTEASEPACKET0
          // request a packet
          if (!getPacket()) {
             return -3;
          }
+# endif // PBTEASEPACKET0
 	 firstPktTime = get_timestamp();
       }
    #ifdef DEBUG
@@ -317,6 +397,12 @@ namespace framexmit {
 	 newseq = oldseq + 1;	// actually, we expect the sequence to start with oldseq
       }
 
+#if defined(PBTEASEPACKET0)
+      if(par.teasePacket0) {
+	 lastSequenceSuccessful = false;
+      }
+#endif // PBTEASEPACKET0
+
       // main receiver loop
       //bool requestPacket = false;	// request a new packet
       bool firstPacket = true;		// first packet received of this seq.?
@@ -345,6 +431,28 @@ namespace framexmit {
                break;
             }
          }
+
+#if defined(PBTEASEPACKET0)
+	 if(par.teasePacket0) {
+	 	////////////////////////////////////////////////////////////
+	 	// Request rebroadcast of packet 0 if we haven't heard anything
+	 	// for 4 seconds
+	 	////////////////////////////////////////////////////////////
+	 	// Only bother with this code if we haven't even received the first packet
+	 	// of the sequence
+	 	if(firstPacket == true) { // true means we're still waiting for first packet
+	 	   // And if there's still nothing coming in
+	 	   if(pkts.empty()) {
+	 	      // Has it now been too long? Should we issue a rebroadcast request?
+	 	      int64_t now = get_timestamp();
+	 	
+	 	      int r = teasePacket0(now, 0);
+	 	      if(r != 0) return(r);
+	 	
+	 	   } // if pkts.empty()
+	 	} // firstPacket == false
+	 } // if par.teasePacket0
+#endif // PBTEASEPACKET0
       
          // calculate difference between pkts[0].header.seq - newseq.
 	 // this is where it loses the first packet. If this is the first packet,
@@ -363,7 +471,16 @@ namespace framexmit {
             // check if sequence out of sync
             if (((diff<0 ? -diff : diff) > maxSequenceOutOfSync) ||
                ((diff >= 1) && (len == 0))) {
-	       newseq = pkts.back()->header.seq + 1;
+	       ////////////////////////////////////////////////////////////
+	       // PB: bug fix for 8 seconds to 4 seconds
+	       ////////////////////////////////////////////////////////////
+	       if(pkts.back()->header.seq != newseq) {
+		  newseq = pkts.back()->header.seq;
+	       }
+	       else {
+		  newseq = pkts.back()->header.seq + 1;
+	       }
+	       ////////////////////////////////////////////////////////////
 	       pkts.clear();
  
                firstPacket = true;
@@ -371,6 +488,20 @@ namespace framexmit {
                len = 0;
                seqDone = false;
 
+#if defined(PBTEASEPACKET0)
+               if(par.teasePacket0) {
+		  // If we get here a new sequence has come in with a new sequence number
+		  // which we are going to take.
+		  // So: don't advance the search start time as we normally do, i.e.
+		  // searchForNewseqStartTime += lastFrameDuration*1000000;
+		  // Since here we have a sequence that just recently arrived, so
+		  // its start time is essentially right now
+		  searchForNewseqStartTime = get_timestamp();
+		  retryDrawOut = 0;
+		  currentFrameDrawingOut = newseq;
+	       }
+#endif // PBTEASEPACKET0
+
                newRetransmissionRate = 0.0;
                if (logison) {
 		  ostringstream msg;
@@ -407,6 +538,11 @@ namespace framexmit {
                   // check if data array is long enough
                   int needed = pkts[0]->header.pktTotal * packetSize;
                   if ((maxlen > 0) && (maxlen < needed)) {
+#if defined(PBTEASEPACKET0)
+		     if(par.teasePacket0) {
+			lastSequenceSuccessful = false;
+		     }
+#endif // PBTEASEPACKET0
                      return -needed;
                   }
                   // allocate data array if necessary
@@ -424,6 +560,11 @@ namespace framexmit {
                   }
                   // check if data array is valid
                   if (data == 0) {
+#if defined(PBTEASEPACKET0)
+		     if(par.teasePacket0) {
+			lastSequenceSuccessful = false;
+		     }
+#endif // PBTEASEPACKET0
                      return -5;
                   }
                   // set sequence, time stamp & duration on caller's request
@@ -462,6 +603,18 @@ namespace framexmit {
                   //  cout << "received last package at " <<
                   //     (TAInow() % 1000000000000) / 1E9 << endl;
                #endif
+#if defined(PBTEASEPACKET0)
+	       if(par.teasePacket0) {
+		  lastFrameDuration = pkts[0]->header.duration;
+		  // Is the last frame duration a reasonable number?
+		  // used to be: if((lastFrameDuration <= 0) || (lastFrameDuration > par.maxFrameDuration)) {
+		  // but lastFrameDuration is unsigned, so ignoring that
+		  if(lastFrameDuration > par.maxFrameDuration) {
+		     lastFrameDuration = par.lastFrameDurationDefault; // duration of last frame defaults to 4 seconds if no frame seen
+		  }
+	       }
+#endif // PBTEASEPACKET0
+
                }
                // remove packet
                pkts.pop_front();
@@ -527,6 +680,11 @@ namespace framexmit {
 
             oldseq = newseq;
             retransmissionRate = newRetransmissionRate;
+#if defined(PBTEASEPACKET0)
+	    if(par.teasePacket0) {
+	       lastSequenceSuccessful = true;
+	    }
+#endif // PBTEASEPACKET0
             return len; // done!
          }
          #ifdef DEBUG
@@ -589,6 +747,19 @@ namespace framexmit {
             retry = 0;
             len = 0;
             seqDone = false;
+#if defined(PBTEASEPACKET0)
+            if(par.teasePacket0) {
+	       // If we got here we (most likely) timed out and are moving on to a new sequence.
+	       // The expected arrival time for the next sequence is the beginning of the last sequence
+	       // plus the expected duration (usually 4 seconds)
+	       // NOT just the time now, i.e. searchForNewseqStartTime = get_timestamp();
+	       // But instead we imagine that the receive() function was called 4 seconds after the
+	       // last start, i.e.
+	       searchForNewseqStartTime += lastFrameDuration*1000000;
+	       retryDrawOut = 0;
+	       currentFrameDrawingOut = newseq;
+	    }
+#endif // PBTEASEPACKET0
             retransmissionRate = newRetransmissionRate;
 
 	    if(!pkts.empty()) {
@@ -650,6 +821,11 @@ namespace framexmit {
          #endif
             rpkt.header.pktLen = n * sizeof (int32_t);
             if (!putPacket (rpkt)) {
+#if defined(PBTEASEPACKET0)
+	       if(par.teasePacket0) {
+		  lastSequenceSuccessful = false;
+	       }
+#endif // PBTEASEPACKET0
                return -6;
             }
          
@@ -819,5 +995,74 @@ namespace framexmit {
       logbuf = "";
    }
 
+#if defined(PBTEASEPACKET0)
+   // top indicates if teasePacket0() being run at top/beginning of receive loop or during loop
+   int frameRecv::teasePacket0 ( const int64_t now, int top)
+   {
+      // Have at least 4 seconds (plus 450ms) elapsed?
+      if(now > searchForNewseqStartTime + lastFrameDuration*1000000 + par.frameDrawRebroadcastInterval * 1000) {
+	 int top_condition = 1; // default to true
+	 if(!top) {
+	    top_condition = retryDrawOut < par.maxRetryDrawOut;
+	 }
+	 // Yes, we should be in the rebroadcast regime. But rebroadcast only if it's been at
+	 // least 450ms since lastrebroadcast (or if there's never been a rebroadcast since 
+	 // starting this sequence) AND if we haven't already used up our retries
+	 if(top_condition && ((retryDrawOut == 0) || 
+			      ((retryDrawOut > 0) && (now > lastRebroadcastTime + par.frameDrawRebroadcastInterval * 1000)))) {
+	    // Increment the drawOutFrameCount counter by one for this sequence if a new sequence
+	    if(currentFrameDrawingOut != lastFrameDrawnOut) {
+	       drawOutFrameCount ++;
+	       lastFrameDrawnOut = currentFrameDrawingOut;
+	    }
+		      
+	    // Have we used up the number of times we have allocated for drawing out a frame?
+	    if(drawOutFrameCount > par.drawOutFrameCountMax) { // defaults to 5
+	       lastSequenceSuccessful = false;
+			 
+	       // Don't return. Just be quiet and don't issue any more retransmit requests
+	    }
+	    else {
+
+	       // Issue a rebroadcast for packet 0 (only)
+	       retransmitpacket	rpkt;
+	       memset (&rpkt.header, 0, sizeof (packetHeader));
+	       rpkt.header.pktType = PKT_REQUEST_RETRANSMIT;
+	       rpkt.header.seq = currentFrameDrawingOut; // the sequence we're expecting
+	       rpkt.header.pktNum = 0;
+	       rpkt.header.pktTotal = 1;
+		          
+	       // Request packet 0 (only)
+	       int n = 0;
+	       int i = 0;
+	       rpkt.pktResend[n++] = i;
+		          
+	       rpkt.header.pktLen = n * sizeof (int32_t);
+	       if (!putPacket (rpkt)) {
+		  lastSequenceSuccessful = false;
+		  return -6;
+	       }
+		          
+	       // Indicate that we've done another retryDrawOut
+	       retryDrawOut ++;
+		          
+	       // Have we used up all of our retries?
+	       if(retryDrawOut >= par.maxRetryDrawOut) {
+		  // Give up on currentFrameDrawingOut and advance
+		  currentFrameDrawingOut ++;
+		  retryDrawOut = 0;
+		  // Advance start time to where we think it should be
+		  searchForNewseqStartTime += lastFrameDuration*1000000;
+	       }
+		          
+	       // Save this new rebroadcast time
+	       lastRebroadcastTime = get_timestamp();
+	    } // if DrawOutFrameCount
+	 } // if retryDrawOut==0 or (retryDrawOut>0 and more than 125ms have passed)
+      } // if now > start + 4 sec
+      return(0);
+   }
+#endif // PBTEASEPACKET0
 
 }
+
