@@ -2,6 +2,7 @@
 #include "wparameters.hh"
 #include "matlab_fcs.hh"
 #include <iostream>
+#include <fstream>
 #include <cstdlib>
 
 using namespace wpipe;
@@ -55,7 +56,10 @@ wparameters::wparameters(void) {
    vetoBandwidthFactor  = 0.5;
    maximumConsistents   = 1000;
 
+   //-----------------------------------  OSC parameters
+   oscStride = 1.0;
 
+   //-----------------------------------  Follow-up parameters
    bayesian = false;
 #ifndef NO_FOLLOWUP
    bayesian = true;
@@ -91,18 +95,18 @@ wparameters::wparameters(const std::string& parFile, int debugLevel) {
    *this = null_parameter_list;
 
    // open parameter file for reading
-   FILE* parameterFileID = fopen(parameterFile.c_str(), "r");
+   ifstream parameterFileID(parameterFile.c_str());
 
    try  {
       read_params(parameterFileID, debugLevel);
       merge_defaults(default_pars);
    } 
    catch (std::exception& e) {
-      fclose(parameterFileID);
+      parameterFileID.close();
       throw;
    }
    // close parameter file
-   fclose(parameterFileID);
+   parameterFileID.close();
 
    // validate
    validate();
@@ -126,20 +130,28 @@ wparameters::buildNetworkString(const str_vect& chan) {
 
 //====================================== Read a single line of a parameter file
 std::string
-wparameters::read_line(FILE* parameterFileID) {
+wparameters::read_line(std::istream& parameterFileID) {
    std::string parameterLine;
-   char temp[1024];
-   char* line = fgets(temp, sizeof(temp), parameterFileID);
-
-   if (line) {
-      // read one line from parameter file
-      parameterLine = line;
+   bool continuation = true;
+   while (continuation && !parameterFileID.fail()) {
+      string line;
+      getline(parameterFileID, line);
+      continuation = false;
+      if (!line.empty()) {
+	 if (line[line.size()-1] == '\\') {
+	    line.erase(line.size()-1,1);
+	    continuation=true;
+	 }
+	 deblank(line);
+	 if (!parameterLine.empty()) parameterLine += " ";
+	 parameterLine += line;
+      }
+   }
 
+   if (!parameterLine.empty()) {
       // remove any comments
-      string::size_type commentIndex = parameterLine.find("#");
-      string::size_type altIndex     = parameterLine.find("%");
-      if (altIndex < commentIndex) commentIndex = altIndex;
-      altIndex                       = parameterLine.find("//");
+      string::size_type commentIndex = parameterLine.find_first_of("#%");
+      string::size_type altIndex     = parameterLine.find("//");
       if (altIndex < commentIndex) commentIndex = altIndex;
       if (commentIndex != string::npos) parameterLine.erase(commentIndex);
 
@@ -151,10 +163,10 @@ wparameters::read_line(FILE* parameterFileID) {
 
 //======================================  Construct parameter list from file.
 int
-wparameters::read_params(FILE* parameterFileID, int debugLevel) {
+wparameters::read_params(std::istream& parameterFileID, int debugLevel) {
    // begin loop over parameter file
    int rc = 1;
-   while (!feof(parameterFileID)) {
+   while (!parameterFileID.eof()) {
       string parameterLine = read_line(parameterFileID);
       // if (empty line, skip to the next line
       if (parameterLine.empty()) continue;
@@ -227,7 +239,24 @@ wparameters::read_params(FILE* parameterFileID, int debugLevel) {
 	 stateMasks = eval(parameterValue);
       } else if (parameterName ==  "errorOnStateError") {
 	 errorOnStateError = strtol(parameterValue.c_str(), 0, 0);
-      } else if (parameterName ==  "injectionNames") {
+      }
+
+      //--------------------------------  OSC condition parameters
+      else if (parameterName == "oscFile") {
+	 oscFile = unquote(parameterValue);
+      }
+      else if (parameterName == "oscFrameType") {
+	 oscFrameType = unquote(parameterValue);
+      }
+      else if (parameterName == "oscConditions") {
+	 oscConditions = eval_str(parameterValue);
+      }
+      else if (parameterName == "oscStride") {
+	 oscStride = strtod(parameterValue.c_str(), 0);
+      }
+      
+      //--------------------------------  Injection parameters
+      else if (parameterName ==  "injectionNames") {
 	 injectionNames = eval_str(parameterValue);
       } else if (parameterName ==  "injectionTypes") {
 	 injectionTypes = eval_str(parameterValue);
@@ -730,6 +759,14 @@ wparameters::display(std::ostream& out) const {
    disp_sarray(out, "stateTypes",          stateTypes);
    disp_darray(out, "stateMasks",          stateMasks);
    disp_bvalue(out, "errorOnStateError",   errorOnStateError);
+
+   //-----------------------------------  Operating state conditions
+   disp_svalue(out, "oscFile",             oscFile);
+   disp_svalue(out, "oscFrameType",        oscFrameType);
+   disp_sarray(out, "oscConditions",       oscConditions);
+   disp_dvalue(out, "oscStride",           oscStride);
+   
+   //-----------------------------------  Injection parameters
    disp_sarray(out, "injectionNames",      injectionNames);
    disp_sarray(out, "injectionTypes",      injectionTypes);
    disp_darray(out, "injectionFactors",    injectionFactors);
