@@ -250,12 +250,13 @@ wpipe::wplot(const plot_chan& pchan) {
    wlog(debugLevel, 2, "  conditioning data");
 
    wcondition conditioner(rsamp, tiling);
-   containers::DFT whitenedData = conditioner.whitenedDFT();
+   containers::DFT whitenedData  = conditioner.whitenedDFT();
+   containers::DFT whitenedCoefs = conditioner.coefficientDFT();
 
    // q transform whitened data
    wlog(debugLevel, 2, "  transforming whitened data");
-   wtransform whitenedTransform(whitenedData, tiling, pchan.outlierFactor, 
-				channelNameString);
+   wtransform whitenedTransform(whitenedData, tiling, whitenedCoefs,
+				pchan.outlierFactor, channelNameString);
    //whitenedTransform.dump(cout);
 
    // identify most significant whitened transform tile
@@ -287,8 +288,8 @@ wpipe::wplot(const plot_chan& pchan) {
  
    //--------------------------------  Q transform raw data
    wlog(debugLevel, 2, "  transforming raw data");
-   wtransform rawTransform(conditioner.rawDFT(), tiling, pchan.outlierFactor, 
-			   channelNameString);
+   wtransform rawTransform(conditioner.rawDFT(), tiling, whitenedCoefs,
+			   pchan.outlierFactor, channelNameString);
    //rawTransform.dump(cout);
 
    weventstack rawEvents, whitenedEvents;
