@@ -664,12 +664,13 @@ wpipe::wscan(const Time& eventTime, const std::string& configFile,
       wlog(debugLevel, 2, "  conditioning data");
 
       wcondition conditioner(rsamp, tiling);
-      containers::DFT whitenedData = conditioner.whitenedDFT();
+      containers::DFT whitenedData  = conditioner.whitenedDFT();
+      containers::DFT whitenedCoefs;
 
       // q transform whitened data
       wlog(debugLevel, 2, "  transforming whitened data");
-      wtransform whitenedTransform(whitenedData, tiling, outlierFactor, 
-				   channelNameString);
+      wtransform whitenedTransform(whitenedData, tiling, whitenedCoefs, 
+				   outlierFactor, channelNameString);
       //whitenedTransform.dump(cout);
 
       // identify most significant whitened transform tile
@@ -713,8 +714,8 @@ wpipe::wscan(const Time& eventTime, const std::string& configFile,
  
       //--------------------------------  Q transform raw data
       wlog(debugLevel, 2, "  transforming raw data");
-      wtransform rawTransform(conditioner.rawDFT(), tiling, outlierFactor, 
-			      channelNameString);
+      wtransform rawTransform(conditioner.rawDFT(), tiling, whitenedCoefs, 
+			      outlierFactor, channelNameString);
       //rawTransform.dump(cout);
 
       weventstack rawEvents, whitenedEvents;
