@@ -9,6 +9,7 @@
 #include "MultiDacc.hh"
 #include "DVecType.hh"
 #include "lcl_array.hh"
+#include "OperStateCondList.hh"
 
 //  constant dInf;
 static const double dInf = 1.0 / 0.0;
@@ -17,7 +18,8 @@ using namespace wpipe;
 
 //======================================  Construct a channel streaming block
 wstream_chan::wstream_chan(const wparameters& par, int dbugLvl) 
-   : pars(par), debugLevel(dbugLvl), minimumTimeShift(0), maximumTimeShift(0)
+   : pars(par), debugLevel(dbugLvl), minimumTimeShift(0), maximumTimeShift(0),
+     oscList(0)
 {
 
    //-----------------------------------  Define tile for Q-transform space
@@ -51,6 +53,28 @@ wstream_chan::wstream_chan(const wparameters& par, int dbugLvl)
       }
    }
 
+   //-----------------------------------  Look for osc overlap
+   size_t nOscConds = pars.oscConditions.size();
+   osc_index.resize(nOscConds, -1);
+   for (size_t i=0; i<nOscConds; ++i) {
+      string cname = pars.oscConditions[i];
+      if (cname.empty() || cname == "NONE") {
+	 continue;
+      }
+      osc_index[i] = i;
+      for (size_t j=0; j<i; ++j) {
+	 if (osc_index[j] < 0) continue;
+	 if (cname == pars.oscConditions[j]) {
+	    osc_index[i] = j;
+	    break;
+	 }
+      }
+      if (debugLevel >= 2) {
+	 cout << "OSC condition " << i << " index " << osc_index[i]
+	      << " conditionName: " << cname << endl;
+      }
+   }
+
    //-----------------------------------  Look for state data overlap
    size_t nStateChans = pars.numberOfStates();
    state_index.resize(nStateChans, -1);
@@ -77,29 +101,66 @@ wstream_chan::wstream_chan(const wparameters& par, int dbugLvl)
 
 //======================================  wstream_chan destructor.
 wstream_chan::~wstream_chan(void) {
+   if (oscList) delete oscList;
+}
+
+//======================================  book a channel in th appropriate mdacc
+void
+wstream_chan::bookInput(MultiDacc& md, const string& channel,
+			const string& frameType) const {
+   //--------------------------------  Split the frame:structure type.
+   string str_type;
+   string fr_type = frameType;
+   size_t cinx = fr_type.find(':');
+   if (cinx != string::npos) {
+      str_type = fr_type.substr(cinx+1);
+      fr_type.erase(cinx);
+   }
+
+   //--------------------------------  Get the stream id.
+   int id = md.frame_type(fr_type);
+   if (id < 0) {
+      cerr << "Channel: " << channel << " frame type: " << fr_type
+	   << " not matched to an input stream. id=0 assumed." << endl;
+      id = 0;
+   }
+   
+   //--------------------------------  Add channel according to type
+   if (str_type == "simdata") {	
+      md.addSimulated(channel.c_str(), id);
+   }
+   else {
+      md.addChannel(channel.c_str(), id);
+   }
 }
 
 //======================================  Request all useful channels.
 void
-wstream_chan::addChannels(MultiDacc& md) const {
-
+wstream_chan::addChannels(MultiDacc& md) {
+   
+   
    //-----------------------------------  Add data channels
    size_t nChannel = pars.channelNames.size();
    for (size_t i=0; i<nChannel; ++i) {
-      string str_type;
-      size_t cinx = pars.frameTypes[i].find(':');
-      if (cinx != string::npos) str_type = pars.frameTypes[i].substr(cinx+1);
-      if (str_type == "simdata") {	
-         md.addSimulated(pars.channelNames[i].c_str(), 0);
-      } else {
-         md.addChannel(pars.channelNames[i].c_str(), 0);
-      }
+      bookInput(md, pars.channelNames[i], pars.frameTypes[i]);
    }
 
+   //-----------------------------------  Add OSC channels
+   if (!pars.oscFile.empty()) {
+      int id = md.frame_type(pars.oscFrameType);
+      if (id < 0) {
+	 cerr << "Unable to find oscFrameType: " << pars.oscFrameType
+	      << ". Using default frame stream." << endl;
+	 id = 0;
+      }
+      oscList = new OperStateCondList(*(md.getDacc(id)));
+      oscList->readConfig(pars.oscFile.c_str());
+   }
+   
    //-----------------------------------  Add state channels
    size_t nStates  = pars.stateNames.size();
    for (size_t i=0; i<nStates; ++i) {
-      md.addChannel(pars.stateNames[i].c_str(), 0);
+      bookInput(md, pars.stateNames[i], pars.stateTypes[i]);
    }
 
    //-----------------------------------  Add request channels
@@ -107,7 +168,7 @@ wstream_chan::addChannels(MultiDacc& md) const {
    for (size_t i=0; i < nInject; ++i) {
       const string& name = pars.injectionNames[i];
       if (!name.empty() && name != "NONE"){
-	 md.addChannel(name.c_str(), 0);
+	 bookInput(md, name, pars.injectionTypes[i]);
       }
    }
 }
@@ -200,7 +261,6 @@ wstream_chan::process(const woutput& outputFiles, const Time& blockStartTime,
       // convert sky coordinate into geocentric coordinate
       wconvertskycoordinates coordinate(pars.skyPosition, blockCenterTime, 
 					pars.skyCoordinateSystem, "geocentric");
-
       if (debugLevel >= 1) {
 	 cout <<  "  sky coordinate:          [" << coordinate[1] << " " 
 	      << coordinate[2] << endl;
@@ -509,28 +569,91 @@ wstream_chan::read_data(MultiDacc& md, const Time& startTime, double duration) {
       error("validChannels not initialized");
    }
 
+   //-----------------------------------  Evaluate osc conditions
+   size_t nOscConds = pars.oscConditions.size();
+   if (nOscConds) wlog(debugLevel, 1, "  getting OSC conditions");
+   else           wlog(debugLevel, 1, "  no OSC conditions");
+   for (size_t i=0; i<nOscConds; i++) {
+      int inx = osc_index[i];
+      if (inx < 0 || !oscList) {
+	 if (debugLevel > 1) {
+	    cout << "   channel[" << i << "] no osc requested." << endl;
+         }
+	 validChannels[i] = true;
+      }
+
+      //--------------------------------  Was condition already evaluated?
+      else if (inx < int(i)) {
+	 if (debugLevel > 1) {
+	    cout << "   channel[" << i << "] state already tested." << endl;
+         }
+	 validChannels[i] = validChannels[inx];
+      }
+
+      //--------------------------------  Fill data with the evaluated condition
+      else {
+	 Time tFill = md.getFillTime();
+	 Interval stride = md.getStride();
+	 size_t nsample = size_t(stride / pars.oscStride);
+	 DVectI idata;
+	 idata.replace_with_zeros(0, 0, nsample);
+	 if (oscList->satisfied(pars.oscConditions[inx])) idata += 1;
+	 TSeries ts(tFill, Interval(pars.oscStride), idata);
+
+	 //--------------------------------  Remove data before this stride.
+	 Time tStart = osc_data[i].getStartTime();
+	 if (!osc_data[i].empty() && tStart < startTime) {
+	    Interval tDelete = startTime - tStart;
+	    osc_data[i].eraseStart(tDelete);
+	 }
+
+	 if (osc_data[i].empty()) {
+	    osc_data[i] = ts;
+	 } else {
+	    osc_data[i].Append(ts);
+	 }
+	 if (osc_data[i].getMinimum() == 1.0 &&
+	     osc_data[i].getMaximum() == 1.0) {
+	    validChannels[i] =true;
+	 } else {
+	    validChannels[i] = false;
+	 }
+      }
+   }
+   
    //-----------------------------------  Evaluate state data
    size_t nStateChans = pars.numberOfStates();
+   std::vector<bool> state_valid(nStateChans, true);
    if (nStateChans) wlog(debugLevel, 1, "  getting state conditions");
    else             wlog(debugLevel, 1, "  no state conditions");
    for (size_t i=0; i<nStateChans; i++) {
-      if (state_index[i] < 0) {
-	 cout << "   channel[" << i << "] no state requested." << endl;
-	 validChannels[i] = true;
-      } else if (state_index[i] < int(i)) {
-	 cout << "   channel[" << i << "] state already tested." << endl;
-	 validChannels[i] = validChannels[state_index[i]];
-      } else {
+      int inx = state_index[i];
+      //--------------------------------  Test whether state was requested
+      if (inx < 0) {
+	 if (debugLevel > 1) {
+	    cout << "   channel[" << i << "] no state requested." << endl;
+         }
+      }
+
+      //--------------------------------  Was state already evaluated?
+      else if (inx < int(i)) {
+	 if (debugLevel > 1) {
+	    cout << "   channel[" << i << "] state already tested." << endl;
+         }
+	 state_valid[i] = state_valid[inx];
+      }
+
+      //--------------------------------  Test specified state
+      else {
 	 string sname = pars.stateNames[i];
 	 int mask = int(pars.stateMasks[i]);
-	 validChannels[i] = true;
 
 	 //--------------------------------  Check for available data.
 	 TSeries* ts = md.refData(sname.c_str());
 	 if (!ts) {
 	    cerr << "Data not available for channel: " << sname[i] << endl;
 	    md.list(cerr);
-	    throw runtime_error("wchunk: Error reading sate channel");
+	    throw runtime_error("wchunk: Error reading state channel");
 	 }
 
 	 //--------------------------------  Remove data before this stride.
@@ -550,15 +673,11 @@ wstream_chan::read_data(MultiDacc& md, const Time& startTime, double duration) {
 	 //--------------------------------  Quick check for missing or invalid 
 	 //                                  state data
 	 TSeries stateData=state_data[i].extract(startTime, duration);
-	 if (stateData.empty() || stateData.getNLess(double(mask))) {
-	    if (debugLevel >= 1) {
-	       cout <<  "    " << sname << ":        data fails quick test "
-		    << "(state < " << mask << ")" << endl; 
-	    }
-	    if (pars.errorOnStateError && stateData.getMinimum() < 0) {
+	 if (stateData.empty()) {
+	    state_valid[i] = false;
+	    if (pars.errorOnStateError) {
 	       error("Error reading detector state");
 	    }
-	    validChannels[i] = false;
 	 }
 
 	 //--------------------------------  Loop over all state data and test
@@ -573,17 +692,18 @@ wstream_chan::read_data(MultiDacc& md, const Time& startTime, double duration) {
 		     cout <<  "    " << sname 
 			  << ":        detector not in requested state" <<endl; 
 		  }
-		  validChannels[i] = false;
+		  state_valid[i] = false;
 		  break;
 	       }
 	    }
-	    if (validChannels[i] && debugLevel >= 2) {
+	    if (state_valid[i] && debugLevel >= 2) {
                cout <<  "    " << sname << ":        state is valid "
                     << "(" << mask << " bits on)" << endl;
             }
   
 	 }
       }
+      validChannels[i] = validChannels[i] && state_valid[i];
    }
 
    //-----------------------------------  Read in the data
@@ -638,6 +758,19 @@ wstream_chan::start_chunk(void) {
       error("Invalid number of state channels");
    }   
    if (state_data.size() != nState) state_data.resize(nState);
+
+   //-----------------------------------   Check state data series
+   size_t nOscCond = pars.oscConditions.size();
+   if (nOscCond > nChan) {
+      error("Invalid number of state channels");
+   }   
+   if (osc_data.size() != nOscCond) osc_data.resize(nOscCond);
+}
+
+//======================================  Thread entry point
+void*
+wstream_chan::thread_entry(void) {
+   return 0;
 }
 
 //======================================  Reset, ready for start of next chunk
