@@ -86,7 +86,7 @@ wtile::init(double timeRange, const dble_vect& qRange,
    /////////////////////////////////////////////////////////////////////////////
 
    // minimum allowable Q prime to prevent window aliasing at zero frequency
-   double minimumAllowableQPrime = 1.0;
+   const double minimumAllowableQPrime = 1.0;
 
    // minimum allowable Q to avoid window aliasing at zero frequency
    double minimumAllowableQ = minimumAllowableQPrime * qPrimeToQ;
