@@ -3,7 +3,7 @@
 #include "wparameters.hh"
 #include "wtile.hh"
 #include "matlab_fcs.hh"
-#include "quick_select.hh"
+//#include "quick_select.hh"
 
 #include "fSeries/DFT.hh"
 #include "dv_function.hh"
@@ -30,9 +30,11 @@ using namespace std;
 
 //====================================== Multi-channel constructor
 wtransform::wtransform(const DFT& indata, const wtile& tiling, 
-		       double outlierFactor, const string& channelName) 
+		       const DFT& coeffs, double outlierFactor, 
+		       const string& channelName) 
 {
-   transforms.push_back(qTransform(indata, tiling, outlierFactor, channelName));
+   qTransform t(indata, tiling, coeffs, outlierFactor, channelName);
+   transforms.push_back(t);
 }
 
 //====================================== Multi-channel constructor
@@ -187,10 +189,10 @@ wtransform::init(const dft_vect& data, const wtile& tiling,
    ////////////////////////////////////////////////////////////////////////////
    if (analysisMode == "independent") {
       transforms.resize(_numberOfChannels);
-      for (size_t chanNumber=0; chanNumber < _numberOfChannels; chanNumber++) {
-	 transforms[chanNumber].transform(data[chanNumber], tiling, 
-					  outlierFactor, 
-					  channelNames[chanNumber]);
+      for (size_t chanNum=0; chanNum < _numberOfChannels; chanNum++) {
+	 transforms[chanNum].transform(data[chanNum], tiling, 
+					  coefficients[chanNum], outlierFactor, 
+					  channelNames[chanNum]);
       }
    }
 
@@ -459,8 +461,9 @@ qTransform::qTransform(void) {
 
 //========================================  qTransform data constructor
 qTransform::qTransform(const containers::DFT& data, const wtile& tiling, 
-		       double outlierFactor, const std::string& chanName) {
-   transform(data, tiling, outlierFactor, chanName);
+		       const containers::DFT& coeffs, double outlierFactor, 
+		       const std::string& chanName) {
+   transform(data, tiling, coeffs, outlierFactor, chanName);
 }
 
 //========================================  qTransform default constructor
@@ -480,7 +483,8 @@ qTransform::dump(std::ostream& out) const {
 //======================================  Perform the q-transform.
 void
 qTransform::transform(const containers::DFT& data, const wtile& tiling, 
-		      double outlierFactor, const std::string& chanName) {
+		      const containers::DFT& whitenCoef, double outlierFactor, 
+		      const std::string& chanName) {
 
    //----------------------------------  Save the channel name.
    _channelName = chanName;
@@ -510,6 +514,11 @@ qTransform::transform(const containers::DFT& data, const wtile& tiling,
 	 TSeries energies(coeffs.getStartTime(), coeffs.getTStep(), dvd);
 
 	 trow& t_pr = _planes[plane]._rows[row];
+	 if (whitenCoef.empty()) {
+	    t_pr.conditioningCoef = 1.0;
+	 } else {
+	    t_pr.conditioningCoef = whitenCoef(tilePlaneRow.frequency).MagSq();
+	 }
 	 t_pr.setNormalized(energies, transientDt, outlierFactor, 
 			    tilePlaneRow);
       }      // end loop over frequency rows
@@ -549,6 +558,8 @@ trow::setNormalized(const TSeries& energies, double transientDt,
    size_t valid_len   = tilePlaneRow.numberOfTiles - 2 * valid_index;
 
    dble_vect sortedEnergies(valid_len, 0.0);
+   dble_iter itr25 = sortedEnergies.begin() + int(0.25*double(valid_len)-0.5);
+   dble_iter itr75 = sortedEnergies.begin() + int(0.75*double(valid_len)-0.5);
    energies.refDVect()->getData(valid_index, valid_len, &sortedEnergies[0]);
 
 #ifndef USE_QUICK_SELECT
@@ -556,19 +567,23 @@ trow::setNormalized(const TSeries& energies, double transientDt,
 
    // identify lower and upper quartile energies
    // Note that in the original matlab
-   double lowerQuartile = sortedEnergies[int(0.25*double(valid_len)-0.5)];
-   double upperQuartile = sortedEnergies[int(0.75*double(valid_len)-0.5)];
+   //double lowerQuartile = sortedEnergies[int(0.25*double(valid_len)-0.5)];
+   //double upperQuartile = sortedEnergies[int(0.75*double(valid_len)-0.5)];
 
 #else
-   quick_select(&sortedEnergies[0], valid_len);
+   //quick_select(&sortedEnergies[0], valid_len);
 
    // identify lower and upper quartile energies
    // Note that in the original matlab
-   int half_len = valid_len/2;
-   double lowerQuartile = quick_select(&sortedEnergies[0], half_len);
-   double upperQuartile = quick_select(&sortedEnergies[(valid_len+1)/2], 
-				       half_len);
+   //int half_len = valid_len/2;
+   //double lowerQuartile = quick_select(&sortedEnergies[0], half_len);
+   //double upperQuartile = quick_select(&sortedEnergies[(valid_len+1)/2], 
+   //				       half_len);
+   nth_element(sortedEnergies.begin(), itr25, sortedEnergies.end());
+   nth_element(itr25+1, itr75, sortedEnergies.end());
 #endif
+   double lowerQuartile = *itr25;
+   double upperQuartile = *itr75;
   
    // determine inter quartile range
    double interQuartileRange = upperQuartile - lowerQuartile;
@@ -576,11 +591,9 @@ trow::setNormalized(const TSeries& energies, double transientDt,
    // energy threshold of outliers
    double outlierThreshold = upperQuartile + outlierFactor*interQuartileRange;
 
-   //-----------------------------------  Mean of valid tile energies
-   int    meanCount = 0;
-   meanEnergy = 0.0;
-
-   //-----------------------------------  Quich sum of lower 3/4ths
+   //-----------------------------------  Mean of valid tile energies. Start 
+   //                                     with a sum of lower 3 quartiles.
+   int meanCount = 0;
    int upper_quartile_index = (valid_len * 3) / 4;
    if (sortedEnergies[upper_quartile_index] <= outlierThreshold) {
       meanCount = upper_quartile_index;
@@ -588,6 +601,7 @@ trow::setNormalized(const TSeries& energies, double transientDt,
    } 
    else {
       upper_quartile_index = 0;
+      meanEnergy = 0.0;
    }
 
    //-----------------------------------  Sum over remainder
