@@ -6,7 +6,7 @@
 //
 //   Added DQ flags in March 2009 - Keith Riles (kriles@umich.edu)
 //
-//   $Id: SenseMonitor.cc 6294 2010-08-13 03:18:16Z john.zweizig@LIGO.ORG $
+//   $Id: SenseMonitor.cc 7395 2015-06-19 01:47:14Z john.zweizig@LIGO.ORG $
 //
 ///////////////////////////////////////////////////////////////////////////
 
@@ -56,7 +56,7 @@
 
 //-->  The next three lines are needed if you are going to generate triggers.
 //     The descriptive title in PIDTITLE should the monitor function.
-#define PIDCVSHDR "$Header: https://redoubt.ligo-wa.caltech.edu/svn/gds/trunk/Monitors/SenseMonitor/SenseMonitor.cc 6294 2010-08-13 03:18:16Z john.zweizig@LIGO.ORG $"
+#define PIDCVSHDR "$Header: https://redoubt.ligo-wa.caltech.edu/svn/gds/trunk/Monitors/SenseMonitor/SenseMonitor.cc 7395 2015-06-19 01:47:14Z john.zweizig@LIGO.ORG $"
 #define PIDTITLE  "Inspiral range monitor"
 #include "ProcIdent.hh"
 
@@ -423,7 +423,8 @@ SenseMonitor::SenseMonitor(int argc, const char *argv[])
 
 	//---------- Get AS_Q channel; decimate to maximum frequency 4096Hz.
 	//           (Keep frequencies above 2kHz for the calibrated PSD dumps.)
-        getDacc().addChannel(Run_Par.mChannel.c_str(), 2);
+	mDecimate.setDecimation(1,4);
+        getDacc().addChannel(Run_Par.mChannel.c_str());
 
         //---------- Initialize FDEasyCalibrate object.  
         //           This should be in monitor constructor (ie, before any data 
@@ -623,17 +624,19 @@ SenseMonitor::ProcessData(void) {
 	    mOutput.cumlog_file_name += ".log";
         }
 
+        //---------- Get AS_Q data to TSeries.
+        TSeries raw_asq = *getDacc().refData(Run_Par.mChannel);
+	if (!mDecimate.isDataValid(raw_asq)) {
+	    cerr << "SensMonitor: decimation filter reset after gap" << endl;
+	    mDecimate.reset();
+	}
+	ts_asq = mDecimate(raw_asq);
+
 	//---------- Things to do once only.
     	if (NStride == 0) {
 
-            //---------- Need some data for initializing certain storage.
-            //           Doesn't matter if IFO is locked.
-
-            //---------- Get AS_Q data to TSeries.
-	    ts_asq = getDacc().refData(Run_Par.mChannel.c_str());
-
 	    //---------- Calcuate PSD.
-	    psd->generate(PSDed_data,ts_asq);
+	    psd->generate(PSDed_data, &ts_asq);
 
 	    //---------- Output run parameters.  
 	    //----- Set starttime
@@ -696,9 +699,6 @@ SenseMonitor::ProcessData(void) {
 
 	//---------- Check that IFO is locked before proceeding.
         if (mOsclist->satisfied(Run_Par.osc_cond.c_str())) {
-	
-	    //---------- Get AS_Q data to TSeries.
-	    ts_asq = getDacc().refData((Run_Par.mChannel).c_str());
 
 	    //---------- Update and apply calibration.  Extracts subsection 
 	    //           of PSD for which we have calibration info.
@@ -711,7 +711,7 @@ SenseMonitor::ProcessData(void) {
             }
 
 	    //---------- Calcuate PSD... Perform calibration before.
-	    psd->generate(PSDed_data, ts_asq, *mCalibrate);
+	    psd->generate(PSDed_data, &ts_asq, *mCalibrate);
             //PSDed_data.Dump(cerr);
 
             //---------- Also retrieve calibration data to report.
@@ -978,9 +978,9 @@ SenseMonitor::ProcessData(void) {
 			cerr << "                     not have units of strain."
 			     << endl;
                     }
-//          	    TimeStr(ts_asq->getStartTime(), time_holder, "%Y%M%d.%H:%N:%S.txt");
-            	    TimeStr(ts_asq->getStartTime(), time_holder, "%s.txt");
-//          	    TimeStr( operator-(ts_asq->getStartTime(),Interval(time_since_update,0)), time_holder, "%s.txt");
+//          	    TimeStr(ts_asq.getStartTime(), time_holder, "%Y%M%d.%H:%N:%S.txt");
+            	    TimeStr(ts_asq.getStartTime(), time_holder, "%s.txt");
+//          	    TimeStr( operator-(ts_asq.getStartTime(),Interval(time_since_update,0)), time_holder, "%s.txt");
        		    mOutput.calpsd_file_name = time_holder;
             	    mOutput.calpsd_file_name = string(mOutput.html_dir) + "/" + Run_Par.IFO + "_CalPSD_GPS_" + mOutput.calpsd_file_name;
             	    mOutput.calpsd_file.open(mOutput.calpsd_file_name.c_str());
