@@ -1,5 +1,5 @@
 /* -*- mode: c++; c-basic-offset: 3 -*- */
-#define PIDCVSHDR "$Header: https://redoubt.ligo-wa.caltech.edu/svn/gds/trunk/Monitors/kleineWelle/kleineWelleM.cc 7347 2015-04-23 18:11:19Z john.zweizig@LIGO.ORG $"
+#define PIDCVSHDR "$Header: https://redoubt.ligo-wa.caltech.edu/svn/gds/trunk/Monitors/kleineWelle/kleineWelleM.cc 7469 2015-09-16 16:26:55Z john.zweizig@LIGO.ORG $"
 #define PIDTITLE  "kleineWelle"
 #include "ProcIdent.hh"
 #include "mywv2lib.hh"
@@ -370,6 +370,26 @@ kleineWelle::kleineWelle(int argc, const char *argv[])
    getDacc().setStride(stride);
    getDacc().setIgnoreMissingChannel(true);
    setStrideAlignment(long(stride), 0.0);
+
+   // set up naming scheme for txt, xml and tren files.
+   string outdir = basename + "-%5r";
+   trg_path.set_directory(outdir);
+   trg_path.set_prefix(basename);
+   trg_path.set_extension("trg");
+   trg_path.set_dt(int(stride));
+   xml_path.set_directory(outdir);
+   xml_path.set_prefix(basename);
+   xml_path.set_extension("xml");
+   xml_path.set_dt(int(stride));
+   trend_path.set_directory(outdir);
+   trend_path.set_prefix(basename+"_TRENDS");
+   trend_path.set_extension("txt");
+   trend_path.set_dt(100000);
+   seg_path.set_directory(outdir);
+   seg_path.set_prefix(segname);
+   seg_path.set_extension("xml");
+   seg_path.set_dt(int(stride));
+
    if (Debug()) cout << "finished configuration setup" << endl;
 }
 
@@ -424,15 +444,19 @@ void kleineWelle::sendRateSeg(rateflag_iter this_rateflag, Time start_sec,
       this_rateflag->channelName.substr(upper_position+1, chan_length - upper_position) << 
       "; threshold " << this_rateflag->sigThreshold << " with rate > " << this_rateflag->rateThreshold << " Hz using "
 	   << stride * (double)(2*this_rateflag->nStride + 1) << " second stride";
-   segname << "DMT-KW_" << this_rateflag->channelName.substr(chan_position+1) << "_RATE" << this_rateflag->sigThreshold << "_"
-	   << this_rateflag->rateThreshold << "HZ_" << stride * (double)(2*this_rateflag->nStride + 1) << "S";
+   segname << "DMT-KW_" << this_rateflag->channelName.substr(chan_position+1)
+	   << "_RATE" << this_rateflag->sigThreshold << "_"
+	   << this_rateflag->rateThreshold << "HZ_"
+	   << stride * (double)(2*this_rateflag->nStride + 1) << "S";
    std::string segmentname = segname.str();
    segmentname.replace(10,1,"_");
    trig::Segment s_kw(segmentname,version,start_sec,end_sec);
    s_kw.setIfos(ifo_name.c_str());
    s_kw.setComment(comment.str().c_str());
    s_kw.setActivity(active);
-   if(Debug()) cout << "sendseg " << segmentname << " " << start_sec << " " << end_sec << " " << active << endl << comment.str() << endl;;
+   if(Debug()) cout << "sendseg " << segmentname << " " << start_sec << " "
+		    << end_sec << " " << active << endl << comment.str()
+		    << endl;;
    lmsg::error_type rc = mSegClient.sendSegment(s_kw);
    if (rc) cout << "Error sending segment: error code " << rc << endl;
 }  /* END: sendRateSeg() */
@@ -452,37 +476,24 @@ void kleineWelle::ProcessData(void)
 
       // check state vector (not done yet)
       if(t0 == lasttime) {
-	 std::stringstream path;
-	 std::stringstream segpath;
-	 path << basename << "-" << t0.getS()/100000;
-	 std::string dir = path.str();
-	 path << "/" << basename << "-" << t0.getS() << "-" 
-	      << stride.GetSecs() << ".trg";
-	 std::string file = path.str();
-	 segpath << dir << "/" << segname << "-" << t0.getS() << "-" 
-                 << stride.GetSecs() << ".xml";
-	 std::string segfile = segpath.str();
-
-	 ofstream outfile;
-	 outfile.open(file.c_str());
-	 if(!outfile) {
-	    mkdir(dir.c_str(), 0755);
-	    outfile.open(file.c_str());
-	    if(!outfile) {
-	       cerr << "cannot create file: " << file;
-	       finish();
-	    }
+	 //----------------------------  Open the output trigger file.
+         trg_path.make_dir(trg_path.dir_name(t0));
+	 string trgfile = trg_path.file_path(t0);
+	 ofstream outfile(trgfile.c_str());
+	 if (!outfile) {
+	    cerr << "cannot write to file: " << trgfile << endl;
+	    finish();
 	 }
 
-	 std::stringstream trendpath;
-	 std::string trendFilename;
-	 ofstream outTrendFile;
-	 trendpath << dir << "/" << basename << "_TRENDS-"
-		   << (int)t0.getS()/100000 << "00000-100000.txt";
-	 outTrendFile.open(trendpath.str().c_str(), ios::app);
+	 //----------------------------  Open the trend file. 
+	 Time::ulong_t tSec = t0.getS();
+	 tSec -= tSec%100000;
 
+	 trend_path.make_dir(trend_path.dir_name(Time(tSec)));
+	 std::string trendpath = trend_path.file_path(Time(tSec));
+	 ofstream outTrendFile(trendpath.c_str(), ios::app);
 	 if(!outTrendFile) {
-	    cerr << "cannot append to file: " << trendpath.str() << endl;
+	    cerr << "cannot append to file: " << trendpath << endl;
 	    finish();
 	 }
 
@@ -513,6 +524,8 @@ void kleineWelle::ProcessData(void)
 	    lastVetoSeg.insert(pair<std::string,Time>(i->first,Time(t0.getS())));
 
 	 outTrendFile << "\n" << t0.getS();
+	 outfile << "# start_time stop_time time frequency unnormalized_energy"
+		 << " normalized_energy chisqdof significance channel" << endl;
 
 	 // process the channels and write the kw veto files
 	 for(channel_iter i=mChanList.begin(); i != mChanList.end(); ++i) {
@@ -569,7 +582,7 @@ void kleineWelle::ProcessData(void)
 		  bool skip = 0;
 		  for(k = 0; k < 2 * this_rateflag->nStride + 1; k++) {
 		     if(this_rateflag->counts[k] == -1) {
-			skip = 1; // rate flag is skipped if we don't h ave enough history
+			skip = 1; // rate flag is skipped if we don't have enough history
 		     }
 		     total += this_rateflag->counts[k];
 		  }
@@ -595,8 +608,9 @@ void kleineWelle::ProcessData(void)
 		  }
 	       }
 	    }
-	    // output ASCII events
-	    copy(events.begin(), events.end(), 
+
+	    // output ASCII triggers
+	    copy(events.begin(), events.end(),
 		 ostream_iterator<Event>(outfile));
 
 	    // loop over the events to construct xml vetoes and triggers
@@ -646,7 +660,14 @@ void kleineWelle::ProcessData(void)
 	    }
 	 }
 
-	 // clean up the veto segments
+	 //----------------------------  close and rename the trigger file
+	 outfile.close();
+	 trg_path.move_to_path(trgfile, t0);
+
+	 //----------------------------  close and rename the trend file
+	 outTrendFile.close();
+
+	 //----------------------------  clean up the veto segments
 	 Time segment_end(t0.getS() + (Time::ulong_t)stride.GetSecs(),0);
 	 for(vetoseg_iter this_veto=mVetoList.begin(); 
 	     this_veto != mVetoList.end(); 
@@ -657,16 +678,22 @@ void kleineWelle::ProcessData(void)
 	    }
 	 }
 
-	 // output XML version
-	 file.replace(file.size() - 3, 3, "xml");
-	 // cout << "writing out XML files " << file << " and " << segfile 
-	 //      << endl;
+	 //-----------------------------  Output XML triggers
+	 xml_path.make_dir(xml_path.dir_name(t0));
+	 string file=xml_path.temp_path(t0);
 	 mTrigClient.setTableFile(file.c_str());
 	 mTrigClient.flush(t0, t0+stride);
+	 xml_path.move_to_path(file, t0);
+
+	 //------------------------------  Output segments
+	 //
+	 //  No renameimg is necessary for the segment file. The s6segment 
+	 //  writer creates a temporary file and renames it after it has 
+	 //  been written.
+	 seg_path.make_dir(seg_path.dir_name(t0));
+	 string segfile=seg_path.file_path(t0);
 	 mSegClient.setTableFile(segfile.c_str());
 	 mSegClient.flush(t0, t0+stride);
-	 outfile.close();
-	 outTrendFile.close();
       } else {
 	 cout << "suppressing trigger output at break in data, expected " 
 	      << lasttime << " found " << t0 << endl;
