@@ -15,7 +15,8 @@
 #include <fstream>
 #include <cstdlib>
 #include <cstdio>
-#include <errno.h>
+#include <cstring>
+#include <cerrno>
 #include <fcntl.h>
 #include <unistd.h>
 #include <sys/socket.h>
@@ -388,7 +389,7 @@ FrameLink::FrameLink(int argc, const char* argv[])
     }
 
     //----------------------------------  Identify myself (after log is open)
-    logstamp(cout) << "Starting framelink: $Id: framelink.cc 7304 2015-03-07 09:03:55Z john.zweizig@LIGO.ORG $" << endl;
+    logstamp(cout) << "Starting framelink: $Id: framelink.cc 7471 2015-09-16 20:00:56Z john.zweizig@LIGO.ORG $" << endl;
     logstamp(cout) << "arguments are:";
     for (int i=1; i<argc; i++) cout << " " << argv[i];
     cout << endl;
@@ -556,6 +557,11 @@ FrameLink::connect_shm(void) {
 	    if (!mOutPart || !mOutPart->attached()) {
 		throw SysError("Failed to attach partition");
 	    }
+	    if (mOutPart->lock(true)) {
+		int lockerr = errno;
+		cerr << "framelink: attempt to lock partition: " << mPartition
+		     << " failed with error: " << strerror(lockerr) << endl;
+	    }
 	}
 	rc = (mOutPart != NULL && mOutPart->attached());
 	break;
