@@ -438,17 +438,29 @@ test_ext(const std::string& file, const std::string& ext) {
     return file.find(ext) == file.size() - ext.size();
 }
 
+static std::string
+get_ext(const std::string& file) {
+    string::size_type inx = file.rfind('.');
+    if (inx == string::npos) return "";
+    if (file.substr(inx) == ".gz") {
+	inx = file.rfind('.', inx-1);
+	if (inx == string::npos) return ".gz";
+    }
+    return file.substr(inx);
+}
+
 //======================================  Read a segment file
 void 
 seg_calc::read_segfile(const string& file, const segID& select, 
 		       const string& fmt) {
     seg_iobase* rdr=0;
+    string file_ext = get_ext(file);
 
     //----------------------------------  Decipher the file type
-    if (test_ext(file, ".xml")) {
+    if (file_ext == ".xml" || file_ext == ".xml.gz") {
 	rdr = new seg_ioxsil();
     }
-    else if (test_ext(file, ".h5")) {
+    else if (file_ext == ".h5") {
 #ifdef HAVE_HDF5
 	rdr = new seg_iohdf5();
 #else
