@@ -19,6 +19,7 @@
 #include "TStyle.h"
 #include <sys/times.h>
 #include <limits.h>
+#include <cmath>
 
 using namespace table;
 using namespace std;
@@ -103,7 +104,7 @@ main(int argc, const char* argv[]) {
 	    verbose = 1;
 	} 
 	else if (argi == "--version") {
-	    cout << "DMT table manipulation utility - $Id: tablepgm.cc 7155 2014-08-23 02:55:50Z john.zweizig@LIGO.ORG $" << endl;
+	    cout << "DMT table manipulation utility - $Id: tablepgm.cc 7410 2015-07-17 08:03:04Z john.zweizig@LIGO.ORG $" << endl;
 	    return 0;
 	} 
 	else if (argi[0] == '-' && argi != "-") {
@@ -314,6 +315,7 @@ exec(istream& in, Table& t, eval::dict_type& d) {
 		va << "-file" = "tablepgm.ps";
 		va << "-fill" = "0";
 		va << "-hold";
+		va << "-logbin" = "false";
 		va << "-nbin" = "100";
 		va << "-title";
 		va << "-rows";
@@ -392,6 +394,17 @@ exec(istream& in, Table& t, eval::dict_type& d) {
 		if (title.empty()) title = va["-eval"] + " histogram";
 		long nbin = long(va.getDouble("-nbin"));
 		TH1D histo("TblPlot", title.c_str(), nbin, xmin, xmax);
+		if (va.getBool("-logbin")) {
+		    double lbin = xmin;
+		    double dbin = exp(log(xmax/xmin)/nbin);
+		    double* bedge = new double[nbin+1];
+		    for (int i=0; i<=nbin; i++) {
+			bedge[i] = lbin;
+			lbin *= dbin;
+		    }
+		    histo.SetBins(nbin, bedge);
+		    delete[] bedge;
+		}
 		if (!va["-xaxis"].empty()) {
 		    TAxis* xax = histo.GetXaxis();
 		    xax->CenterTitle(kTRUE);
